<?php
/**
 * $JA#COPYRIGHT$
 */
defined('_JEXEC') or die;

class HikashopFilterHelper
{
	protected $menuid = '';
	
	function getLangSuffix()
	{
		$langs =  JFactory::getLanguage()->getKnownLanguages();
		$lang_sfx = array();
		foreach ($langs as $lang) {
			$lang_sfx[] = str_replace('-', '_', strtolower($lang['tag']));
		}
		return $lang_sfx;
	}
	
	function getFilterItems($catid)
	{
		$filterItems = array();
		$lang_sfx = $this->getLangSuffix();
		foreach ($lang_sfx AS $lang) {
			$filterItems[$lang] = $this->getItemList($catid, $lang);
		}
		return $filterItems;
	}
	
	function getItemList($catid, $lang)
	{
		$itemList = new stdCLass();
		$catList = $this->getCatList($catid, $lang);
		if (empty($catList)) {
			return array();
		}
		$baseItemList = $this->getBaseItemList( $catList );

		foreach ($baseItemList as $bItem) {
			$item = $this->getItem($bItem, $lang);
			if (!empty($item)) {
				$property = 'item_'.$item->id;
				$itemList->{ $property } = $item;
			}
		}
		return $itemList;
	}
	
	function getBaseItemList($catList)
	{
		$array_id = array();
		foreach ($catList as $cat)
		{
			$array_id[] = $cat->category_id;
		}
		$q = 'select b.* from #__hikashop_product_category as a';
		$q .= ' left join #__hikashop_product as b on (a.product_id = b.product_id)';
		$q .= ' where a.category_id in ('.  implode(',', $array_id).') and b.product_published = 1';
		$q .= ' group by a.product_id';
		$q .= ' order by a.product_id desc';
		$db = JFactory::getDbo()->setQuery($q);
		return $db->loadObjectList();
	}
	
	function getItem($baseItem, $lang)
	{
		$item = new stdClass();
		$item->id = $baseItem->product_id;
		$item->name = $baseItem->product_name;
		if ($baseItem->product_quantity) 
			$item->is_salable = 1;
		
		$item->weight =  (float) $baseItem->product_weight ? $baseItem->product_weight:0;
		$item->width = (float) $baseItem->product_width ? $baseItem->product_width : 0 ;
		$item->length = (float) $baseItem->product_length ? $baseItem->product_length : 0;
		$item->height = (float) $baseItem->product_height ? $baseItem->product_height : 0;
		$item->price = (float) $this->getPrice($baseItem)->price;
		$item->frontend_price = $this->getPrice($baseItem)->frontend_price;
		$item->rating = (float) $baseItem->product_average_score;
		$item->width_rating = $baseItem->product_average_score * 20;
		$item->created_date = $baseItem->product_created;
		$item->modified_date = $baseItem->product_modified;
		$this->attr = array();
		$this->addBadgeInfo($baseItem);
		$this->addCategoryInfo($baseItem, $lang);
		$this->addBrandInfo($baseItem, $lang);
		$this->addTagInfo($baseItem, $lang);
		$this->addCharacteristics($baseItem, $lang);
		$item->attr = $this->attr;

		$item->url = $this->getUrl($baseItem, $lang);
		$item->menuid = $this->menuid;
		$item->thumbnail = $this->getThumbnail($baseItem);
		return $item;
	}
	
	function getUrl($baseItem, $lang)
	{
		// use cache
		if (empty($this->menuid))
			$this->menuid = $this->getMenuId($lang);
		
		$url = 'index.php?option=com_hikashop&ctrl=product&task=show&cid='.$baseItem->product_id.'&name='.$baseItem->product_alias.$this->menuid;
		return $url;
	}
	
	function getMenuId($lang)
	{
		$helper = new $this;
		$explode = explode('_', $lang);
		$lang_tag = $explode[0].'-'.strtoupper($explode[1]);
		$q = 'select id, params from #__menu';
		$q .= ' where ( link = "index.php?option=com_hikashop&view=product&layout=listing" or link = "index.php?option=com_hikashop&view=category&layout=listing")';
		$q .= ' and ( language ="'.$lang_tag.'" or language ="*" ) and published = 1';
		$q .= ' order by link desc';
		$db = JFactory::getDbo()->setQuery($q);
		$helper->menu_list = $db->loadObjectList();
		
		$num_cats_of_menu = array();
		foreach ($helper->menu_list as $menu)
		{
			$helper->catList = array();
			$helper->childCatList = array();
			$params = json_decode($menu->params);
			if (!empty($params->hk_product->category)) 
				$num_cats_of_menu[$menu->id] = count($helper->getChildCatList($params->hk_product->category, $lang));
			elseif (!empty($params->hk_category->category)) 
				$num_cats_of_menu[$menu->id] = count($helper->getChildCatList($params->hk_category->category, $lang));
			else 
				$num_cats_of_menu[$menu->id] = 0;
		}

		if (!empty($num_cats_of_menu)) {
			$menuId = array_search( max($num_cats_of_menu), $num_cats_of_menu );
			return '&Itemid='.$menuId;
		} else 
			return '';
	}
	
	function getThumbnail($baseItem)
	{
		$q = 'select file_path from #__hikashop_file';
		$q .= ' where file_type="product" and file_ref_id = '.$baseItem->product_id;
		$q .= ' order by file_ordering';
		$db = JFactory::getDbo()->setQuery($q);
		$file_path = $db->loadResult();
		if (!empty($file_path)) {
			$imageHelper = hikashop_get('helper.image');
			$thumbnail = $imageHelper->getThumbnail($file_path);
			$thumbnail_url =  substr($thumbnail->url, 3);
		} else {
			$thumbnail_url = 'media/com_hikashop/images/barcode.png';
		}
		
		return $thumbnail_url;
	}
	
	function addCharacteristics($baseItem, $lang)
	{
		$ids = $this->getVariantProductId($baseItem);
		array_unshift( $ids, $baseItem->product_id );

		$q  = 'select a.*,b.* from '.hikashop_table('variant').' as a';
		$q .= ' inner join '.hikashop_table('characteristic').' as b on a.variant_characteristic_id=b.characteristic_id';
		$q .= ' where a.variant_product_id in ('.implode(',',$ids).')';
		$q .= ' group by b.characteristic_id';
		$db = JFactory::getDbo()->setQuery($q);
		$result = $db->loadObjectList();
		$characteristics = array();
		
		if(empty($result)) return;
		
		foreach( $result as $value)
		{
			if ($value->characteristic_parent_id == 0) {
				$characteristics[ 'char'.$value->characteristic_id ] = array();
			}
		}
		
		foreach( $characteristics as $key => $char )
		{
			foreach( $result as $value )
			{
				if ($value->characteristic_parent_id == (int) substr($key, 4)) {
					$this->attr[$key]['value'][] = $value->characteristic_id;
					$this->attr[$key]['frontend_value'][] = $value->characteristic_value;
				}
			}
		}
		
		return $this->attr;
	}
	
	function getVariantProductId($baseItem)
	{
		$q = 'select product_id from #__hikashop_product';
		$q .= ' where product_published = 1 and product_type = "variant" and product_parent_id ='. (int) $baseItem->product_id;
		$db = JFactory::getDbo()->setQuery($q);
		return $db->loadColumn();
	}
	
	function addTagInfo($baseItem, $lang)
	{
		$helperTags = new JHelperTags();
		$tags = $helperTags->getItemTags('com_hikashop.product',$baseItem->product_id, true);
		foreach ( $tags as $tag)
		{
			$this->attr['tag']['value'][] = $tag->tag_id;
			$this->attr['tag']['frontend_value'][] = $tag->title;
		}
		return $this->attr;
	}
	
	function addBrandInfo($baseItem, $lang)
	{
		if ($baseItem->product_manufacturer_id) {
			$q = 'select * from #__hikashop_category';
			$q .= ' where category_type = "manufacturer" and category_id ='. (int) $baseItem->product_manufacturer_id;
			$q .= ' and category_published = 1';
			$db = JFactory::getDbo()->setQuery($q);
			$result = $db->loadObject();
			$this->attr['brand']['value'][] = $result->category_id;
			$this->attr['brand']['frontend_value'][] = $result->category_name;
		}
		return $this->attr;
	}
	
	function addCategoryInfo($baseItem, $lang)
	{
		$productClass = hikashop_get('class.product');
		$catIds = $productClass->getCategories($baseItem->product_id);
		$catTree = $this->getCatTree($this->catList);
		foreach( $catTree as $singleCat)
		{
			if (in_array($singleCat->category_id, $catIds)) {
				$this->attr['category']['value'][] = $singleCat->category_id;
				$this->attr['category']['frontend_value'][] = $singleCat->category_name;
			}
		}
		return $this->attr;
	}
	
	function addBadgeInfo($baseItem)
	{
		$badgeClass = hikashop_get('class.badge');
		$badgeClass->loadBadges($baseItem);
		if ( !empty($baseItem->badges) ) {
			foreach ($baseItem->badges as $badge) {
				$this->attr['badges']['value'][] = $badge->badge_id;
				$this->attr['badges']['frontend_value'][] = $badge->badge_name;
			}
		}
		return	$this->attr;
	}
	
	function getPrice($baseItem)
	{
		$pprice = new stdClass();
		$currency_id = hikashop_getCurrency();
		$currencyClass = hikashop_get('class.currency');
		$config = hikashop_config();
		$main_currency = (int) $config->get('main_currency', 1);
		$zone_id = hikashop_getZone(null);
		$discount_before_tax = (int) $config->get('discount_before_tax', 0);
		$ids = array($baseItem->product_id);
		$currencyClass->getPrices($baseItem, $ids, $currency_id, $main_currency, $zone_id, $discount_before_tax);
		$prices = $baseItem->prices;
		if (empty($prices)) {
			$pprice->price = 0;
			$pprice->frontend_price = JText::_('COM_JAMEGAFILTER_FREE');
		} else {
			$price_quantity = array();
			foreach ($prices as $price) {
				$price_quantity[$price->price_min_quantity] = $price->price_value_with_tax;
			}
			$min_price = min(array_values($price_quantity));
			$pprice->price = $min_price;
			$currency = $this->getCurrency($currency_id);
			$pformated = $currencyClass->format($pprice->price, $currency_id);
			$pprice->frontend_price = str_replace($currency, '<span class="currency">' . $currency . '</span>', $pformated);
		}

		return $pprice;
	}

	function getCurrency($currency_id) {
		$q = 'SELECT * FROM `#__hikashop_currency` WHERE currency_id ='.$currency_id;
		$db = JFactory::getDbo()->setQuery($q);
		$obj = $db->loadObject();
		switch ($obj->currency_format) {
			case '%n':
				return $obj->currency_symbol;
			
			case '%i':
				return $obj->currency_code;
		}
	}
	
	function getCatList($catid, $lang)
	{
		$this->catList = array();
		$this->childCatList = array();
		$q = 'select *';
		$q .= ' from #__hikashop_category';
		$q .= ' where category_type="product" and category_published = 1 and category_id ='. (int) $catid;
		$db = JFactory::getDbo()->setQuery($q);
		$result = $db->loadObject();
		$this->catList = $this->getChildCatList($catid, $lang);
		if($catid != 1)
			array_unshift($this->catList, $result);
		return $this->catList;
	}
	
	
	function getChildCatList ($catid, $lang)
	{
		$q = 'select *';
		$q .= ' from #__hikashop_category ';
		$q .= ' where category_type = "product" and category_parent_id = ' .(int) $catid;
		$q .= ' and category_published = 1';
		$db = JFactory::getDbo();
		$db->setQuery($q);
		$list = $db->loadObjectList();
		foreach ( $list as $item )
		{
			$this->childCatList[] = $item;
			$this->getChildCatList($item->category_id, $lang);
		}
		return $this->childCatList;
	}
	
	function getCatTree($catList)
	{
		$catTree = array();
		foreach ( $catList as $cat )
		{
			$nametree = '';
			$tcat = clone $cat;
			foreach( $catTree as $singleCat ) 
			{
				if ($cat->category_parent_id == $singleCat->category_id) {
					$nametree = $singleCat->category_name.' &raquo; '.$nametree;
					break;
				} 
			}

			$tcat->category_name = $nametree . $cat->category_name;
			array_push($catTree, $tcat);
		}
		return $catTree;
	}
}